use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.007005
use Test::Spelling 0.12;
use Pod::Wordlist;


add_stopwords(<DATA>);
all_pod_files_spelling_ok( qw( bin lib ) );
__DATA__
Array
BSON
Binary
Bloor
Bool
Bytes
Code
DBPointer
DBRef
Daniels
David
Decimal128
Doc
Double
Duclos
Eric
Finn
Golden
Gunn
Inc
Int32
Int64
Kostyuk
Leich
MaxKey
MinKey
MongoDB
OID
OIDs
ObjectID
ObjectId
Oleg
Olivier
PCRE
PP
Pat
Petr
Písař
Raw
Regex
Reis
Riedel
Robert
Sedlacek
Stefan
String
Symbol
Thomas
Time
Timestamp
Tobias
Types
VERSIONING
Wallace
Yury
Zavarin
and
bson
codec
codecs
cub
david
dbrefs
email
eric
ixhash
lib
minimalist
numification
odc
oid
pgunn
ppisar
rs
sharding
tbsliver
tock
toyou1995
wallace
yury
